/* DataClass.java

	Purpose:
		
	Description:
		
	History:
		Mon Dec 02 17:43:46 CST 2019, Created by rudyhuang

Copyright (C) 2019 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart;

import org.zkoss.chart.util.DynamicalAttribute;

/**
 * A data class or range for the choropleth map.
 *
 * @author rudyhuang
 * @since 7.2.1.0
 */
public class DataClass extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		color, from, name, to
	}

	/**
	 * Returns the color of each data class.
	 * If not set, the color is pulled from the global or chart-specific colors array.
	 * <p>
	 * Default: null.
	 */
	public Color getColor() {
		return (Color) getAttr(Attrs.color, null).asValue();
	}

	/**
	 * Sets the color of each data class.
	 */
	public void setColor(Color color) {
		setAttr(Attrs.color, color);
	}

	/**
	 * Sets the color of each data class.
	 */
	public void setColor(String color) {
		setColor(new Color(color));
	}

	/**
	 * Sets the color of each data class.
	 */
	public void setColor(LinearGradient color) {
		setColor(new Color(color));
	}

	/**
	 * Sets the color of each data class.
	 */
	public void setColor(RadialGradient color) {
		setColor(new Color(color));
	}

	/**
	 * Returns the start of the value range that the data class represents, relating to the point value.
	 * The range of each dataClass is closed in both ends, but can be overridden by the next dataClass.
	 * <p>
	 * Default: null
	 */
	public Number getFrom() {
		return getAttr(Attrs.from, null).asNumber();
	}

	/**
	 * Sets the start of the value range that the data class represents, relating to the point value.
	 * The range of each dataClass is closed in both ends, but can be overridden by the next dataClass.
	 */
	public void setFrom(Number from) {
		setAttr(Attrs.from, from);
	}

	/**
	 * Returns the name of the data class as it appears in the legend.
	 * If no name is given, it is automatically created based on the from and to values.
	 * For full programmatic control, legend.labelFormatter can be used.
	 * In the formatter, this.from and this.to can be accessed.
	 * <p>
	 * Default: null
	 */
	public String getName() {
		return getAttr(Attrs.name, null).asString();
	}

	/**
	 * Sets the name of the data class as it appears in the legend.
	 * If no name is given, it is automatically created based on the from and to values.
	 * For full programmatic control, legend.labelFormatter can be used.
	 * In the formatter, this.from and this.to can be accessed.
	 */
	public void setName(String name) {
		setAttr(Attrs.name, name);
	}

	/**
	 * Returns the end of the value range that the data class represents, relating to the point value.
	 * The range of each dataClass is closed in both ends, but can be overridden by the next dataClass.
	 * <p>
	 * Default: null
	 */
	public Number getTo() {
		return getAttr(Attrs.to, null).asNumber();
	}

	/**
	 * Sets the end of the value range that the data class represents, relating to the point value.
	 * The range of each dataClass is closed in both ends, but can be overridden by the next dataClass.
	 */
	public void setTo(Number to) {
		setAttr(Attrs.to, to);
	}
}
