/* DataLabelsTextPath.java

	Purpose:
		
	Description:
		
	History:
		Thu Nov 21 18:00:35 CST 2019, Created by rudyhuang

Copyright (C) 2019 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart.plotOptions;

import java.util.LinkedHashMap;
import java.util.Map;

import org.zkoss.chart.Optionable;
import org.zkoss.chart.PlotAttribute;
import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.chart.util.MapsHelper;
import org.zkoss.lang.Generics;

/**
 * Options for a label text which should follow marker's shape.
 * Note: Only SVG-based renderer supports this option.
 *
 * @author rudyhuang
 * @since 7.2.1.0
 */
public class DataLabelsTextPath extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		attributes, enabled
	}

	/**
	 * Returns the presentation attributes for the text path, if any. Otherwise, create a new one.
	 * @param <V> Value type
	 */
	public <V> Map<String, V> getAttributes() {
		if (!containsKey(Attrs.attributes)) {
			setAttributes(new LinkedHashMap<String, V>());
		}
		return Generics.cast(getAttr(Attrs.attributes));
	}

	/**
	 * Sets the presentation attributes for the text path.
	 * @param <V> Value type
	 */
	public <V> void setAttributes(Map<String, V> attributes) {
		setAttr(Attrs.attributes, attributes, NOT_NULL_VALUE);
	}

	/**
	 * Sets the presentation attributes for the text path.
	 * This method allows <b>simple</b> settings like
	 * <pre>{@code
	 * textPath.setAttributes("dy: 12; startOffset: '45%'; textLength: 80");
	 * }</pre>
	 * <p>
	 * In the format of key:value, and ";" separating multiple key:value pairs,
	 * if you have more a complex attributes, we recommend using {@link #setAttributes(Map)}
	 * @param style one or many key:value pairs in a single line string format
	 */
	public void setAttributes(String style) {
		setAttributes(MapsHelper.parse(new LinkedHashMap<String, String>(), style, ':', ';', '\''));
	}

	/**
	 * Returns enable or disable textPath option for link's or marker's data labels.
	 * <p>
	 * Default: false
	 */
	public boolean isEnabled() {
		return getAttr(Attrs.enabled, false).asBoolean();
	}

	/**
	 * Sets enable or disable textPath option for link's or marker's data labels.
	 */
	public void setEnabled(boolean enabled) {
		setAttr(Attrs.enabled, enabled);
	}
}
