/* PackedBubblePlotOptions.java

	Purpose:
		
	Description:
		
	History:
		Wed Nov 20 11:48:15 CST 2019, Created by rudyhuang

Copyright (C) 2019 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart.plotOptions;

import org.zkoss.chart.PlotAttribute;
import org.zkoss.chart.util.DynamicalAttribute;

/**
 * An option for packed bubble plot.
 * <p>
 * All the options in this class support {@link DynamicalAttribute}.
 *
 * @author rudyhuang
 * @since 7.2.1.0
 */
public class PackedBubblePlotOptions extends BubblePlotOptions {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		dataLabels, // override
		layoutAlgorithm,
		useSimulation
	}

	@Override
	public PackedBubbleDataLabels getDataLabels() {
		PackedBubbleDataLabels dataLabels = (PackedBubbleDataLabels) this.getAttr(Attrs.dataLabels);
		if (dataLabels == null) {
			dataLabels = new PackedBubbleDataLabels();
			setDataLabels(dataLabels);
		}
		return dataLabels;
	}

	@Override
	public void setDataLabels(DataLabels dataLabels) {
		setAttr(Attrs.dataLabels, dataLabels);
	}

	/**
	 * Return options for layout algorithm when simulation is enabled, if any. Otherwise, create a new one.
	 */
	public PackedBubbleLayoutAlgorithm getLayoutAlgorithm() {
		PackedBubbleLayoutAlgorithm layoutAlgorithm = (PackedBubbleLayoutAlgorithm) getAttr(Attrs.layoutAlgorithm);
		if (layoutAlgorithm == null) {
			layoutAlgorithm = new PackedBubbleLayoutAlgorithm();
			setLayoutAlgorithm(layoutAlgorithm);
		}
		return layoutAlgorithm;
	}

	/**
	 * Sets options for layout algorithm when simulation is enabled.
	 */
	public void setLayoutAlgorithm(PackedBubbleLayoutAlgorithm layoutAlgorithm) {
		setAttr(Attrs.layoutAlgorithm, layoutAlgorithm);
	}

	/**
	 * Returns an option is giving a possibility to choose between using simulation
	 * for calculating bubble positions.
	 * These reflects in both animation and final position of bubbles.
	 * Simulation is also adding options to the series graph based on used layout.
	 * In case of big data sets, with any performance issues, it is possible to
	 * disable animation and pack bubble in a simple circular way.
	 * <p>
	 * Default: true
	 */
	public boolean isUseSimulation() {
		return getAttr(Attrs.useSimulation, true).asBoolean();
	}

	/**
	 * Sets an option is giving a possibility to choose between using simulation
	 * for calculating bubble positions.
	 * These reflects in both animation and final position of bubbles.
	 * Simulation is also adding options to the series graph based on used layout.
	 * In case of big data sets, with any performance issues, it is possible to
	 * disable animation and pack bubble in a simple circular way.
	 */
	public void setUseSimulation(boolean useSimulation) {
		setAttr(Attrs.useSimulation, useSimulation);
	}
}
