/* XRangePlotOptions.java

	Purpose:
		
	Description:
		
	History:
		Mon Dec 09 16:02:04 CST 2019, Created by rudyhuang

Copyright (C) 2019 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart.plotOptions;

import org.zkoss.chart.PlotAttribute;
import org.zkoss.chart.util.DynamicalAttribute;

/**
 * An option for xrange plot
 * <p>
 * All the options in this class support {@link DynamicalAttribute}.
 *
 * @author rudyhuang
 * @since 7.2.1.0
 */
public class XRangePlotOptions extends ColumnPlotOptions {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		partialFill
	}

	/**
	 * Returns a partial fill for each point, if any. Otherwise, create a new one.
	 * Typically used to visualize how much of a task is performed.
	 * The partial fill object can be set either on series or point level.
	 */
	public XRangePartialFill getPartialFill() {
		XRangePartialFill partialFill = (XRangePartialFill) getAttr(Attrs.partialFill);
		if (partialFill == null) {
			partialFill = new XRangePartialFill();
			setPartialFill(partialFill);
		}
		return partialFill;
	}

	/**
	 * Sets a partial fill for each point.
	 * Typically used to visualize how much of a task is performed.
	 * The partial fill object can be set either on series or point level.
	 */
	public void setPartialFill(XRangePartialFill partialFill) {
		setAttr(Attrs.partialFill, partialFill);
	}

	/**
	 * Sets a partial fill with an amount for each point.
	 * Typically used to visualize how much of a task is performed.
	 * The partial fill object can be set either on series or point level.
	 */
	public void setPartialFill(Number amount) {
		setPartialFill(new XRangePartialFill(amount));
	}
}
